#!/usr/bin/python3
# -- coding: UTF-8 --

import syslog
import dbus
from pydbus import SystemBus
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio

class ListControl():
    """
    <node>
        <interface name='com.kylin.ListControl'>
             <method name='SetWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetWhiteList' >
                <arg type='as' direction='out' name='white-list'/>
            </method>
            <method name='ClearWhiteList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='SetBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackList' >
                <arg type='as' direction='out' name='black-list'/>
            </method>
            <method name='ClearBlackList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.sdk = SystemBus().get("com.kylin.kysdk.ListControl",
                                   "/com/kylin/kysdk/ListControl")
        except Exception:
            print("get sdk dbus error")
        

    def SetWhiteList(self, pid: str, vid: str) -> int:
        syslog.syslog(syslog.LOG_DEBUG, "listctl SetWhiteList pid:{} vid:{}".format(pid, vid))
        try:
            rc = self.sdk.SetWhiteList(pid, vid)
            syslog.syslog(syslog.LOG_DEBUG, "listctl SetWhiteList rc:{}".format(rc))
        except Exception:
            return -1
        return rc

    def GetWhiteList(self) -> list:
        syslog.syslog(syslog.LOG_DEBUG, "listctl GetWhiteList")
        try:
            rc = self.sdk.GetWhiteList()
            syslog.syslog(syslog.LOG_DEBUG, "listctl GetWhiteList rc:{}".format(rc))
        except Exception:
            return []
        return rc

    def ClearWhiteList(self) -> int:
        syslog.syslog(syslog.LOG_DEBUG, "listctl ClearWhiteList")
        try:
            rc = self.sdk.ClearWhiteList()
            syslog.syslog(syslog.LOG_DEBUG, "listctl ClearWhiteList rc:{}".format(rc))
        except Exception:
            return -1
        return rc

    def SetBlackList(self, pid: str, vid: str) -> int:
        syslog.syslog(syslog.LOG_DEBUG, "listctl SetBlackList pid:{} vid:{}".format(pid, vid))
        try:
            rc = self.sdk.SetBlackList(pid, vid)
            syslog.syslog(syslog.LOG_DEBUG, "listctl SetBlackList rc:{}".format(rc))
        except Exception:
            return -1
        return rc


    def GetBlackList(self) -> list:
        syslog.syslog(syslog.LOG_DEBUG, "listctl GetBlackList")
        try:
            rc = self.sdk.GetBlackList()
            syslog.syslog(syslog.LOG_DEBUG, "listctl GetBlackList")
        except Exception:
            return []
        return rc

    def ClearBlackList(self) -> int:
        syslog.syslog(syslog.LOG_DEBUG, "listctl ClearBlackList")
        try:
            rc = self.sdk.ClearBlackList()
            syslog.syslog(syslog.LOG_DEBUG, "listctl ClearBlackList")
        except Exception:
            return -1
        return rc


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.ListControl", ListControl())
    loop.run()
