#!/usr/bin/python3
# -- coding: UTF-8 --

import dbus
import dbus.service
from pydbus import SystemBus
from pydbus import SessionBus
import syslog
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

BUS_NAME = "com.kylin.volumectl"
OBJ_NAME = "/com/kylin/volumectl"
IFACE = "mdm.sound.control"

class SessSoundControl(dbus.service.Object):
    """
        <node>
            <interface name='mdm.sound.control'>
                <method name='SetVolume'>
                <arg type='i' name='nValue' direction='in'/>
                <arg type='i' name='ret' direction='out'/>
                </method>
                <method name='GetVolume'>
                <arg type='i' name='status' direction='out'/>
                </method>
                <method name='DecreaseVolume'>
                <arg type='i' name='nValue' direction='in'/>
                <arg type='i' name='ret' direction='out'/>
                </method>
                <method name='IncreaseVolume'>
                <arg type='i' name='nValue' direction='in'/>
                <arg type='i' name='ret' direction='out'/>
                </method>
            </interface>
        </node>
    """

    def __init__(self):
        try:
            print("__init__  SessSoundControl")

            bus = dbus.SessionBus()
            bus.request_name(BUS_NAME)
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, OBJ_NAME)

            self.PulseAudio = SessionBus().get("org.ukui.media","/org/ukui/media")
        except Exception:
            print("get sdk dbus error")

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i")
    def SetVolume(self,nValue):
        syslog.syslog(syslog.LOG_DEBUG, "SessSound SetVolume nValue:{}".format(nValue))
        try:
            ret = 0

            outRet = self.PulseAudio.SetOutputVolume(nValue)
            inRet = self.PulseAudio.SetInputVolume(nValue)
            if outRet == -1 or inRet == -1:
                ret = -1

            syslog.syslog(syslog.LOG_DEBUG, "SessSound SetVolume nValue:{} rc:{} outRet:{} inRet:{}"
            .format(nValue, ret, outRet, inRet))
        except Exception:
            return -1
        return ret

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="i")
    def GetVolume(self):
        syslog.syslog(syslog.LOG_DEBUG, "SessSound GetVolume")
        try:
            ret = 0
            ret = self.PulseAudio.GetOutputVolume()

            syslog.syslog(syslog.LOG_DEBUG, "SessSound GetVolume ret:{}".format(ret))
        except Exception:
            return -1
        return ret

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i")
    def DecreaseVolume(self,nValue):
        syslog.syslog(syslog.LOG_DEBUG, "SessSound DecreaseVolume nValue:{}".format(nValue))
        try:
            ret = -1

            #检测输入值是否符合要求
            if(nValue < 0 or nValue > 100):
                return -1

            # 如果降低的音量大于当前音量，就设置音量为0
            volume = self.PulseAudio.GetOutputVolume()
            if nValue > volume:
                ret = self.PulseAudio.SetOutputVolume(0)
            else:
                ret = self.PulseAudio.SetOutputVolume(volume - nValue)

            syslog.syslog(syslog.LOG_DEBUG, "SessSound DecreaseVolume nValue:{} ret:{}"
            .format(nValue, ret))
        except Exception:
            return -1
        return ret

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i")
    def IncreaseVolume(self,nValue):
        syslog.syslog(syslog.LOG_DEBUG, "SessSound IncreaseVolume nValue:{}".format(nValue))
        try:
            ret = -1
            
            #检测输入值是否符合要求
            if(nValue < 0 or nValue > 100):
                return -1

            # 如果降低的音量大于当前音量，就设置音量为0
            volume = self.PulseAudio.GetOutputVolume()
            if nValue + volume > 100:
                ret = self.PulseAudio.SetOutputVolume(100)
            else:
                ret = self.PulseAudio.SetOutputVolume(volume + nValue)

            syslog.syslog(syslog.LOG_DEBUG, "SessSound IncreaseVolume nValue:{} ret:{}"
            .format(nValue, ret))
        except Exception:
            return -1
        return ret


if __name__ == '__main__':
    a = SessSoundControl()

    loop = GLib.MainLoop()
    loop.run()