#!/usr/bin/python3
# -- coding: UTF-8 --

import dbus
import dbus.service
from pydbus import SystemBus
import dbus.mainloop.glib
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio
import ctypes
from ctypes import *
import os
import syslog

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

BUS_NAME = "com.kylin.mdm"
OBJ_NAME = "/com/kylin/mdm/WirelessDeviceCtl/BluetoothCtl"
IFACE = "com.kylin.mdm.WirelessDeviceCtl.BluetoothCtl"

class BluetoothControl(dbus.service.Object):
    def __init__(self):
        try:
            print("__init__  BluetoothControl")

            bus = dbus.SystemBus()
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, OBJ_NAME)

            self.so = ctypes.CDLL("/usr/lib/kysdk-security-adaptor/libkybluetoothsec.so")
            
            if(os.path.exists("/etc/kysdk-security-adaptor/") == False):
                os.mkdir("/etc/kysdk-security-adaptor/")

            self.dbus = SystemBus().get("org.freedesktop.DBus",
                                   "/org/freedesktop/DBus")

        except Exception:
            print("get sdk dbus error")

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="b", out_signature="i")
    def SetDeviceEnableRoot(self, bEnable: bool) -> int:
        strEnable = 0
        if bEnable == True:
            strEnable = "1"
        if bEnable == False:
            strEnable = "0"

        try:
            rc = -1
            with open("/etc/kysdk-security-adaptor/bt-device-enable-root", "w") as file:
                file.write(strEnable)
                rc = 0 
                syslog.syslog(syslog.LOG_DEBUG, "bt SetDeviceEnableRoot strEnable:{} bEnable:{}".format(strEnable, bEnable))

        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "SetDeviceEnableRoot error")
            return -1

        return rc

    @dbus.service.method(dbus_interface=IFACE,
                        in_signature="ias", out_signature="i",sender_keyword='sender')
    def SetListCtl(self, nType:int, strList:list, sender=None) -> int: 
        try:
            cuid = self.dbus.GetConnectionUnixUser(sender)
            enableRoot =  self.GetDeviceEnableRoot()
            syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl cuid:{} enableRoot:{} sender:{}".format(cuid, enableRoot, sender))

            if(enableRoot == True and cuid != 0):
                syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl err right")
                return  -1

            rc = -1
            if nType != 1 and nType != 2:
                syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl err type nType:{}".format(nType))
                return -1

            #先清空黑白名单，再添加
            if nType == 1:
                rc = self.so.kdk_adapter_device_clear_bluetooth_adapter_whitelist()
                syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl white nType:{} rc:{}".format(nType, rc))
                if rc != 0:
                    return -1

                for strMac in strList:
                    rc = self.so.kdk_adapter_device_add_bluetooth_adapter_whitelist(ctypes.c_char_p(bytes(strMac, encoding="utf8")))
                    syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl white strMac:{} rc:{}".format(strMac, rc))
                    if rc != 0:
                        rc = -1
                        break
            
            if nType == 2:
                rc = self.so.kdk_adapter_device_clear_bluetooth_adapter_blacklist()
                syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl black nType:{} rc:{}".format(nType, rc))
                if rc == -1:
                    return -1

                for strMac in strList:
                    rc = self.so.kdk_adapter_device_add_bluetooth_adapter_blacklist(ctypes.c_char_p(bytes(strMac, encoding="utf8")))
                    syslog.syslog(syslog.LOG_DEBUG, "bt SetListCtl black strMac:{} rc:{}".format(strMac, rc))
                    if rc != 0:
                        rc = -1
                        break 


        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "SetListCtl error")
            return -1

        return rc       

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i", sender_keyword='sender')
    def SetEnableListCtl(self, nType:int, sender=None) -> int:
        try:
            rc = -1
            cuid = self.dbus.GetConnectionUnixUser(sender)
            enableRoot = self.GetDeviceEnableRoot()
            syslog.syslog(syslog.LOG_DEBUG, "bt SetEnableListCtl cuid:{} enableRoot:{} sender:{}".format(cuid, enableRoot, sender))
                
            if(enableRoot == True and cuid != 0):
                syslog.syslog(syslog.LOG_DEBUG, "bt SetEnableListCtl err right")
                return  -1
            
            if nType != 0 and nType != 1 and nType != 2:
                syslog.syslog(syslog.LOG_DEBUG, "bt SetEnableListCtl err type nType:{}".format(nType))
                return -1

            bt_mode = 0
            if nType == 0:
                bt_mode = 0
            if nType == 1:
                bt_mode = 2
            if nType == 2:
                bt_mode = 1

            rc = self.so.kdk_adapter_device_set_bluetooth_adapter_bwm(bt_mode)

            syslog.syslog(syslog.LOG_DEBUG, "bt SetEnableListCtl nType:{} bt_mode:{} rc:{}"
            .format(nType, bt_mode, rc))

            if rc != 0:
                rc = -1
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "SetEnableListCtl error")
            return -1
        return rc          

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="i")
    def GetDeviceEnableRoot(self) -> int:
        if os.path.exists("/etc/kysdk-security-adaptor/bt-device-enable-root") == False:
            syslog.syslog(syslog.LOG_DEBUG, "bt GetDeviceEnableRoot bt-device-enable-root not exist")
            return False

        try:
            bEnable = False
            with open("/etc/kysdk-security-adaptor/bt-device-enable-root", "r") as file:
                rc = file.readline()
                if rc == "1":
                    bEnable = True
                if rc == "0":
                    bEnable = False
                syslog.syslog(syslog.LOG_DEBUG, "bt GetDeviceEnableRoot bEnable rc:{}".format(bEnable, rc))
                
        except Exception:
            syslog.syslog(syslog.LOG_DEBUG, "GetDeviceEnableRoot error")
            return False
        return bEnable

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="i")
    def GetEnableListCtl(self) -> int:
        try:
            rc = -1
            bt_mode = self.so.kdk_adapter_device_get_bluetooth_adapter_bwm()
            syslog.syslog(syslog.LOG_DEBUG, "bt GetEnableListCtl bt_mode:{}".format(bt_mode))

            if bt_mode == 0:
                rc = 0
            if bt_mode == 1:
                rc = 2
            if bt_mode == 2:
                rc = 1

        except Exception:
            print("GetListCtl error")
            return -1
        return rc              

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="ias")
    def GetListCtl(self, nType):
        try: 
            ret = 0
            res_list=[]

            if nType != 1 and nType != 2:
                syslog.syslog(syslog.LOG_DEBUG, "bt GetListCtl err type nType:{}".format(nType))
                return -1,res_list

            len = ctypes.c_int(0)
            if nType == 1:
                self.so.kdk_adapter_device_get_bluetooth_adapter_whitelist.restype = POINTER(ctypes.c_char_p)
                plist = self.so.kdk_adapter_device_get_bluetooth_adapter_whitelist(byref(len))
            if nType == 2:
                self.so.kdk_adapter_device_get_bluetooth_adapter_blacklist.restype = POINTER(ctypes.c_char_p)
                plist = self.so.kdk_adapter_device_get_bluetooth_adapter_blacklist(byref(len))


            for i in range(0, len.value):
                res_list.append(str(plist[i], "UTF-8"))

            syslog.syslog(syslog.LOG_DEBUG, "bt GetListCtl res_list:{} ".format(res_list))
        except Exception:
            print("GetListCtl error")
            return -1,res_list
        return ret,res_list  

if __name__ == '__main__':
    a = BluetoothControl()

    loop = GLib.MainLoop()    
    loop.run()