#!/usr/bin/python3
# -- coding: UTF-8 --

import syslog
import dbus
import dbus.service
from pydbus import SystemBus
import dbus.mainloop.glib
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

BUS_NAME = "com.kylin.mdm"
OBJ_NAME = "/com/kylin/mdm/UsbCtl"
IFACE = "com.kylin.mdm.UsbCtl"

class UsbControl(dbus.service.Object):
    """
    <node>
        <interface name="com.kylin.mdm.UsbCtl">
            <method name="SetEnable">
                <arg type="i" direction="out"/>
                <arg name="nType" type="i" direction="in"/>
                <arg name="bEnable" type="b" direction="in"/>
            </method>
            <method name="GetEnable">
                <arg type="b" direction="out"/>
                <arg name="nType" type="i" direction="in"/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            print("__init__  UsbControl")

            bus = dbus.SystemBus()
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, OBJ_NAME)

            self.sdk = SystemBus().get("com.kylin.kysdk.netcardcontrol",
                                   "/com/kylin/kysdk/netcardcontrol")
        except Exception:
            print("get sdk dbus error")


    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="ib", out_signature="i")
    def SetEnable(self, nType: int, bEnable: bool) -> int:
        syslog.syslog(syslog.LOG_DEBUG, "netcard SetEnable nType:{} bEnable:{}".format(nType, bEnable))
        try:
            if nType != 1:
                return -1

            rc = -1
            if bEnable == True:
                status = 1
            if bEnable == False:
                status = 0

            rc = self.sdk.SetNetcardStatus(status)
            syslog.syslog(syslog.LOG_DEBUG, "netcard SetEnable rc:{}".format(rc))
        
        except Exception:
            return -1
        return rc  

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i")
    def GetEnable(self, nType: int) -> int:
        try:
            if nType != 1:
                return False

            rc = False
            ret = self.sdk.GetNetcardStatus()
            if ret == 1:
                rc = True
            syslog.syslog(syslog.LOG_DEBUG, "netcard GetEnable ret:{} rc:{}".format(ret,rc))

        except Exception:
            return False
        return rc


if __name__ == '__main__':
    a = UsbControl()
    
    loop = GLib.MainLoop()
    loop.run()