#!/usr/bin/python3
# -- coding: UTF-8 --

from ctypes import sizeof
from pickle import FALSE
import dbus
import dbus.service
from pydbus import SystemBus
import dbus.mainloop.glib
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, Gtk, Gio
import os
import syslog

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

BUS_NAME = "com.kylin.mdm"
OBJ_NAME = "/com/kylin/mdm/WirelessDeviceCtl/WiFiCtl"
IFACE = "com.kylin.mdm.WirelessDeviceCtl.WiFiCtl"

class WifiControl(dbus.service.Object):
    """
    <node>
        <interface name="com.kylin.mdm.WirelessDeviceCtl.WiFiCtl">
            <method name="SetDeviceEnableRoot">
                <arg type="i" direction="out"/>
                <arg name="bEnable" type="b" direction="in"/>
            </method>
            <method name="GetDeviceEnableRoot">
                <arg type="b" direction="out"/>
                </method>
            <method name="SetListCtl">
                <arg type="i" direction="out"/>
                <arg name="nType" type="i" direction="in"/>
                <arg name="strList" type="as" direction="in"/>
            </method>
            <method name="SetEnableListCtl">
                <arg type="i" direction="out"/>
                <arg name="nType" type="i" direction="in"/>
            </method>
            <method name="GetEnableCtlList">
                <arg type="i" direction="out"/>
            </method>
            <method name="GetListCtl">
                <arg type="i" direction="out"/>
                <arg name="nType" type="i" direction="in"/>
                <arg name="strList" type="as" direction="out"/>
            </method>
            <method name="SetDisconnectRoot">
                <arg type="i" direction="out"/>
                <arg name="bEnable" type="b" direction="in"/>
            </method>
            <method name="GetDisconnectRoot">
                <arg type="i" direction="out"/>
                <arg name="bEnable" type="b" direction="out"/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            print("__init__ WifiControl")

            bus = dbus.SystemBus()
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, OBJ_NAME)

            self.sdk = SystemBus().get("com.kylin.kysdk.devicesec",
                                   "/com/kylin/kysdk/devicesec/wlan/wifi")
            if(os.path.exists("/etc/kysdk-security-adaptor/") == False):
                os.mkdir("/etc/kysdk-security-adaptor/")

            self.dbus = SystemBus().get("org.freedesktop.DBus",
                                   "/org/freedesktop/DBus")

        except Exception:
            print("get sdk dbus error")

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="b", out_signature="i")
    def SetDeviceEnableRoot(self, bEnable: bool) -> int:
        strEnable = 0
        if bEnable == True:
            strEnable = "1"
        if bEnable == False:
            strEnable = "0"

        try:
            rc = -1
            with open("/etc/kysdk-security-adaptor/wifi-device-enable-root", "w") as file:
                file.write(strEnable)
                rc = 0
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetDeviceEnableRoot rc:{} strEnable:{}".format(rc, strEnable))
        
        except Exception:
            return -1
        return rc

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="b")
    def GetDeviceEnableRoot(self) -> bool:
        if os.path.exists("/etc/kysdk-security-adaptor/wifi-device-enable-root") == False:
            return False

        try:
            bEnable = False
            with open("/etc/kysdk-security-adaptor/wifi-device-enable-root", "r") as file:
                rc = file.readline()
                if rc == "1":
                    bEnable = True
                if rc == "0":
                    bEnable = False
            syslog.syslog(syslog.LOG_DEBUG, "wifi GetDeviceEnableRoot rc:{} bEnable:{}".format(rc, bEnable))
        except Exception:
            return False
        return bEnable  

    @dbus.service.method(dbus_interface=IFACE,
                        in_signature="ias", out_signature="i",sender_keyword='sender')
    def SetListCtl(self, nType: int, strList: list, sender=None) -> int:
        try: 
            rc = -1
            cuid = self.dbus.GetConnectionUnixUser(sender)
            enableRoot =  self.GetDeviceEnableRoot()
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetListCtl cuid:{} enableRoot:{} sender:{}".format(cuid, enableRoot, sender))

            if(enableRoot == True and cuid != 0):
                return  -1

            if nType == 1:
                for strMac in strList:
                    rc = self.sdk.SetWhitelist(strMac)
                    if rc != 0:
                        rc = -1
                        break
            if nType == 2:
                for strMac in strList:
                    rc = self.sdk.SetBlacklist(strMac)
                    if rc != 0:
                        rc = -1
                        break
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetListCtl nType:{} strList:{} rc:{}".format(nType, strList, rc))

        except Exception:
            return -1
        return rc       

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="i", sender_keyword='sender')
    def SetEnableListCtl(self, nType:int, sender=None) -> int:
        try:        
            cuid = self.dbus.GetConnectionUnixUser(sender)
            enableRoot =  self.GetDeviceEnableRoot()
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetEnableListCtl cuid:{} enableRoot:{} sender:{}".format(cuid, enableRoot, sender))

            if(enableRoot == True and cuid != 0):
                return  -1

            status = 0
            if nType == 1:
                status = 2
            if nType == 2:
                status = 1

            rc = self.sdk.SetMode(status)
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetEnableListCtl nType:{} status:{} rc:{}".format(nType, status, rc))
            if rc != 0:
                rc = -1
        except Exception:
            return -1
        return rc        
    
    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="i")
    def GetEnableCtlList(self) -> int:
        try:
            rc = self.sdk.GetMode()
            syslog.syslog(syslog.LOG_DEBUG, "wifi GetEnableCtlList rc:{}".format(rc))

            if rc == 1:
                rc = 2
            if rc == 2:
                rc = 1
        except Exception:
            return -1
        return rc              
    
    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="i", out_signature="ias")
    def GetListCtl(self, nType):
        try: 
            ret = -1
            res_list = []
            if nType == 1:
                ret,res_list = self.sdk.GetWhitelist()
            if nType == 2:
                ret,res_list = self.sdk.GetBlacklist()
            
            if ret == len(res_list):
                print("GetListCtl len:%d" %len(res_list))
                ret = 0
            syslog.syslog(syslog.LOG_DEBUG, "wifi GetListCtl nType:{} res_list:{} ret:{}".format(nType, res_list, ret))

        except Exception:
            return -1
        return ret,res_list  

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="b", out_signature="i")
    def SetDisconnectRoot(self, bEnable: bool) -> int:
        if bEnable == True:
            strEnable = "1"
        if bEnable == False:
            strEnable = "0"

        try:
            rc = -1
            with open("/etc/kysdk-security-adaptor/wifi-device-discon-root", "w") as file:
                file.write(strEnable)
                rc = 0
            syslog.syslog(syslog.LOG_DEBUG, "wifi SetDisconnectRoot bEnable:{} strEnable:{} rc:{}".format(bEnable, strEnable, rc))

        except Exception:
            return -1
        return rc

    @dbus.service.method(dbus_interface=IFACE,
                         in_signature="", out_signature="b")
    def GetDisconnectRoot(self):
        if os.path.exists("/etc/kysdk-security-adaptor/wifi-device-discon-root") == False:
            return False

        try:
            rc = -1
            bEnable = False
            with open("/etc/kysdk-security-adaptor/wifi-device-discon-root", "r") as file:
                tmpRet = file.readline()
                rc = 0
                if tmpRet == "1":
                    bEnable = True
                if tmpRet == "0":
                    bEnable = False
            syslog.syslog(syslog.LOG_DEBUG, "wifi GetDisconnectRoot bEnable:{} tmpRet:{} rc:{}".format(bEnable, tmpRet, rc))

        except Exception:
            return FALSE
        return  bEnable


if __name__ == '__main__':
    a = WifiControl()

    loop = GLib.MainLoop()
    loop.run()